"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.RedisThrottleAndUsageStorageManager = void 0;
const server_services_utils_1 = require("@fluidframework/server-services-utils");
const server_services_telemetry_1 = require("@fluidframework/server-services-telemetry");
/**
 * Manages storage of throttling metrics and usage data in redis.
 * @internal
 */
class RedisThrottleAndUsageStorageManager {
    constructor(redisClientConnectionManager, parameters) {
        this.redisClientConnectionManager = redisClientConnectionManager;
        this.expireAfterSeconds = 60 * 60 * 24;
        this.prefix = "throttle";
        if (parameters?.expireAfterSeconds) {
            this.expireAfterSeconds = parameters.expireAfterSeconds;
        }
        if (parameters?.prefix) {
            this.prefix = parameters.prefix;
        }
        redisClientConnectionManager.getRedisClient().on("error", (error) => {
            server_services_telemetry_1.Lumberjack.error("Throttle Manager Redis Error", { [server_services_telemetry_1.CommonProperties.telemetryGroupName]: "throttling" }, error);
        });
    }
    async setThrottlingMetric(id, throttlingMetric) {
        const throttlingKey = this.getKey(id);
        return (0, server_services_utils_1.executeRedisMultiWithHmsetExpire)(this.redisClientConnectionManager.getRedisClient(), throttlingKey, throttlingMetric, this.expireAfterSeconds);
    }
    async setThrottlingMetricAndUsageData(throttlingId, throttlingMetric, usageStorageId, usageData) {
        const throttlingKey = this.getKey(throttlingId);
        const usageDataString = JSON.stringify(usageData);
        server_services_telemetry_1.Lumberjack.info(`Pushing usage data - id: ${usageStorageId}, data: ${usageDataString}}`, {
            [server_services_telemetry_1.BaseTelemetryProperties.tenantId]: usageData.tenantId,
            [server_services_telemetry_1.BaseTelemetryProperties.documentId]: usageData.documentId,
            [server_services_telemetry_1.CommonProperties.clientId]: usageData.clientId,
        });
        return (0, server_services_utils_1.executeRedisMultiWithHmsetExpireAndLpush)(this.redisClientConnectionManager.getRedisClient(), throttlingKey, throttlingMetric, usageStorageId, usageDataString, this.expireAfterSeconds);
    }
    async getThrottlingMetric(id) {
        const throttlingMetric = await this.redisClientConnectionManager
            .getRedisClient()
            .hgetall(this.getKey(id));
        if (Object.keys(throttlingMetric).length === 0) {
            return undefined;
        }
        // All values retrieved from Redis are strings, so they must be parsed
        return {
            count: Number.parseInt(throttlingMetric.count, 10),
            lastCoolDownAt: Number.parseInt(throttlingMetric.lastCoolDownAt, 10),
            throttleStatus: throttlingMetric.throttleStatus === "true",
            throttleReason: throttlingMetric.throttleReason,
            retryAfterInMs: Number.parseInt(throttlingMetric.retryAfterInMs, 10),
        };
    }
    async setUsageData(id, usageData) {
        const usageDataString = JSON.stringify(usageData);
        server_services_telemetry_1.Lumberjack.info(`Pushing usage data - id: ${id}, data: ${usageDataString}}`, {
            [server_services_telemetry_1.BaseTelemetryProperties.tenantId]: usageData.tenantId,
            [server_services_telemetry_1.BaseTelemetryProperties.documentId]: usageData.documentId,
            [server_services_telemetry_1.CommonProperties.clientId]: usageData.clientId,
        });
        await this.redisClientConnectionManager.getRedisClient().lpush(id, usageDataString);
    }
    async getUsageData(id) {
        const usageDataString = await this.redisClientConnectionManager.getRedisClient().rpop(id);
        if (usageDataString) {
            return JSON.parse(usageDataString);
        }
        return undefined;
    }
    getKey(id) {
        return `${this.prefix}:${id}`;
    }
}
exports.RedisThrottleAndUsageStorageManager = RedisThrottleAndUsageStorageManager;
//# sourceMappingURL=redisThrottleAndUsageStorageManager.js.map